<?php

namespace Assetproject\Report\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Assetproject\Report\Http\Requests\ReportAdminApiRequest;
use Assetproject\Report\Interfaces\ReportRepositoryInterface;
use Assetproject\Report\Models\Report;

/**
 * Admin API controller class.
 */
class ReportAdminApiController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.api';

    /**
     * Initialize report controller.
     *
     * @param type ReportRepositoryInterface $report
     *
     * @return type
     */
    public function __construct(ReportRepositoryInterface $report)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:admin.api');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $report;
        parent::__construct();
    }

    /**
     * Display a list of report.
     *
     * @return json
     */
    public function index(ReportAdminApiRequest $request)
    {
        $reports  = $this->repository
            ->pushCriteria(new \Assetproject\Report\Repositories\Criteria\ReportAdminCriteria())
            ->setPresenter('\\Assetproject\\Report\\Repositories\\Presenter\\ReportListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $reports['code'] = 2000;
        return response()->json($reports) 
                         ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display report.
     *
     * @param Request $request
     * @param Model   Report
     *
     * @return Json
     */
    public function show(ReportAdminApiRequest $request, Report $report)
    {
        $report         = $report->presenter();
        $report['code'] = 2001;
        return response()->json($report)
                         ->setStatusCode(200, 'SHOW_SUCCESS');;

    }

    /**
     * Show the form for creating a new report.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(ReportAdminApiRequest $request, Report $report)
    {
        $report         = $report->presenter();
        $report['code'] = 2002;
        return response()->json($report)
                         ->setStatusCode(200, 'CREATE_SUCCESS');

    }

    /**
     * Create new report.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(ReportAdminApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $report          = $this->repository->create($attributes);
            $report          = $report->presenter();
            $report['code']  = 2004;

            return response()->json($report)
                             ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
;
        }
    }

    /**
     * Show report for editing.
     *
     * @param Request $request
     * @param Model   $report
     *
     * @return json
     */
    public function edit(ReportAdminApiRequest $request, Report $report)
    {
        $report         = $report->presenter();
        $report['code'] = 2003;
        return response()-> json($report)
                        ->setStatusCode(200, 'EDIT_SUCCESS');;
    }

    /**
     * Update the report.
     *
     * @param Request $request
     * @param Model   $report
     *
     * @return json
     */
    public function update(ReportAdminApiRequest $request, Report $report)
    {
        try {

            $attributes = $request->all();

            $report->update($attributes);
            $report         = $report->presenter();
            $report['code'] = 2005;

            return response()->json($report)
                             ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the report.
     *
     * @param Request $request
     * @param Model   $report
     *
     * @return json
     */
    public function destroy(ReportAdminApiRequest $request, Report $report)
    {

        try {

            $t = $report->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('report::report.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
