<?php

namespace Assetproject\Report\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetproject\Report\Http\Requests\ReportAdminWebRequest;
use Assetproject\Report\Interfaces\ReportRepositoryInterface;
use Assetproject\Report\Models\Report;

/**
 * Admin web controller class.
 */
class ReportAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize report controller.
     *
     * @param type ReportRepositoryInterface $report
     *
     * @return type
     */
    public function __construct(ReportRepositoryInterface $report)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $report;
        parent::__construct();
    }

    /**
     * Display a list of report.
     *
     * @return Response
     */
    public function index(ReportAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $reports  = $this->repository
            ->pushCriteria(new \Assetproject\Report\Repositories\Criteria\ReportAdminCriteria())
                ->setPresenter('\\Assetproject\\Report\\Repositories\\Presenter\\ReportListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $reports['recordsTotal']    = $reports['meta']['pagination']['total'];
            $reports['recordsFiltered'] = $reports['meta']['pagination']['total'];
            $reports['request']         = $request->all();
            return response()->json($reports, 200);

        }

        $this   ->theme->prependTitle(trans('report::report.names').' :: ');
        return $this->theme->of('report::admin.report.index')->render();
    }

    /**
     * Display report.
     *
     * @param Request $request
     * @param Model   $report
     *
     * @return Response
     */
    public function show(ReportAdminWebRequest $request, Report $report)
    {
        if (!$report->exists) {
            return response()->view('report::admin.report.new', compact('report'));
        }

        Form::populate($report);
        return response()->view('report::admin.report.show', compact('report'));
    }

    /**
     * Show the form for creating a new report.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReportAdminWebRequest $request)
    {

        $report = $this->repository->newInstance([]);

        Form::populate($report);

        return response()->view('report::admin.report.create', compact('report'));

    }

    /**
     * Create new report.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReportAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $report          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('report::report.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/report/report/'.$report->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show report for editing.
     *
     * @param Request $request
     * @param Model   $report
     *
     * @return Response
     */
    public function edit(ReportAdminWebRequest $request, Report $report)
    {
        Form::populate($report);
        return  response()->view('report::admin.report.edit', compact('report'));
    }

    /**
     * Update the report.
     *
     * @param Request $request
     * @param Model   $report
     *
     * @return Response
     */
    public function update(ReportAdminWebRequest $request, Report $report)
    {
        try {

            $attributes = $request->all();

            $report->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('report::report.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/report/report/'.$report->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/report/report/'.$report->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the report.
     *
     * @param Model   $report
     *
     * @return Response
     */
    public function destroy(ReportAdminWebRequest $request, Report $report)
    {

        try {

            $t = $report->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('report::report.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/report/report/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/report/report/'.$report->getRouteKey()),
            ], 400);
        }
    }
}
